<?php

namespace Drupal\aegir_site_subscriptions_recurly\WebhookNotificationHandlers;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\aegir_site_subscriptions\Wrappers\SubscriptionWrapper;
use Drupal\Core\Messenger\MessengerInterface;
use GuzzleHttp\Client;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Processes Recurly notifications for Aegir.
 */
abstract class WebhookNotificationHandler {

  const NOTIFICATION_MAP = [
    'new_account_notification' => 'NewAccountWebhookNotificationHandler',
    'canceled_account_notification' => 'CanceledAccountWebhookNotificationHandler',
    'billing_info_updated_notification' => 'UpdatedAccountWebhookNotificationHandler',
    'reactivated_account_notification' => 'ReactivatedAccountWebhookNotificationHandler',
    'new_invoice_notification' => 'NewInvoiceWebhookNotificationHandler',
    'processing_invoice_notification' => 'ProcessingInvoiceWebhookNotificationHandler',
    'closed_invoice_notification' => 'ClosedInvoiceWebhookNotificationHandler',
    'past_due_invoice_notification' => 'OverdueInvoiceWebhookNotificationHandler',
    'new_subscription_notification' => 'NewSubscriptionWebhookNotificationHandler',
    'updated_subscription_notification' => 'UpdatedSubscriptionWebhookNotificationHandler',
    'canceled_subscription_notification' => 'CanceledSubscriptionWebhookNotificationHandler',
    'expired_subscription_notification' => 'ExpiredSubscriptionWebhookNotificationHandler',
    'renewed_subscription_notification' => 'RenewedSubscriptionWebhookNotificationHandler',
    'scheduled_payment_notification' => 'ScheduledPaymentWebhookNotificationHandler',
    'processing_payment_notification' => 'ProcessingPaymentWebhookNotificationHandler',
    'successful_payment_notification' => 'SuccessfulPaymentWebhookNotificationHandler',
    'failed_payment_notification' => 'FailedPaymentWebhookNotificationHandler',
    'successful_refund_notification' => 'SuccessfulRefundPaymentWebhookNotificationHandler',
    'void_payment_notification' => 'VoidPaymentWebhookNotificationHandler',
  ];

  /**
   * Account information.
   *
   * @var \SimpleXMLElement|null
   */
  protected $account;

  /**
   * Subscription information.
   *
   * @var \Drupal\aegir_site_subscriptions\Wrappers\SubscriptionWrapper|null
   */
  protected $subscription;

  /**
   * Transaction information.
   *
   * @var \SimpleXMLElement|null
   */
  protected $transaction;

  /**
   * Invoice information.
   *
   * @var \SimpleXMLElement|null
   */
  protected $invoice;

  protected $result = FALSE;

  /**
   * The logging service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The current HTTP/S request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * A Guzzle http client instance.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The configuration service.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configuration;

  /**
   * The user messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $userMessenger;

  /**
   * Factory method.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container this instance should use.
   * @param array $data
   *   Notification data.
   *
   * @see ContainerInjectionInterface::create()
   *
   * @return \Drupal\aegir_site_subscriptions_recurly\WebhookNotificationHandlers\WebhookNotificationHandler
   */
  public static function create(ContainerInterface $container, array $data) {
    return new static(
      $data,
      $container->get('logger.factory')->get('aegir_site_subscriptions_recurly'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('module_handler'),
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('messenger')
    );
  }

  /**
   * Class constuctor.
   *
   * @param array $data
   *   The notification data.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logging service.
   * @param \Symfony\Component\HttpFoundation\Request $current_request
   *   The current HTTP/S request.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP/S client.
   * @param \Drupal\Core\Config\ConfigFactory $configuration
   *   The configuration service.
   * @param \Drupal\Core\Messenger\MessengerInterface $user_messenger
   *   The module handler.
   */
  public function __construct(
    array $data,
    LoggerInterface $logger,
    Request $current_request,
    ModuleHandlerInterface $module_handler,
    Client $http_client,
    ConfigFactory $configuration,
    MessengerInterface $user_messenger
  ) {
    $this->account = $data['account'];
    $this->transaction = $data['transaction'];
    $this->invoice = $data['invoice'];
    $this->logger = $logger;
    $this->currentRequest = $current_request;
    $this->moduleHandler = $module_handler;
    $this->httpClient = $http_client;
    $this->configuration = $configuration;
    $this->userMessenger = $user_messenger;

    if (empty($data['subscription']->uuid)) {
      $this->subscription = NULL;
    }
    else {
      $this->subscription = SubscriptionWrapper::getInstance((string) $data['subscription']->uuid);
    }
  }

  /**
   * Handle a webhook notification from Recurly.
   *
   * Must be overriden by subclasses to actually do something.
   *
   * @return $this
   *   The object itself.
   */
  abstract public function handleNotification();

  /**
   * Get the processing result.
   *
   * @return bool
   *   TRUE if processing was successful; FALSE otherwise.
   */
  public function getResult() {
    return $this->result;
  }

  /**
   * Fetch the remote account code.
   *
   * @return string
   *   The remote account code.
   */
  protected function getAccountCode() {
    return (string) $this->account->account_code;
  }

  /**
   * Log a warning message and return the current object.
   *
   * @param $message
   *   The message to log.
   *
   * @return $this
   *   The object itself.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  protected function failAndLogMessage($message) {
    $this->logger->warning($message, [
      '%uuid' => $this->subscription->getId(),
      '%user' => $this->subscription->getDrupalUserId($this->getAccountCode()),
    ]);

    $this->result = FALSE;
    return $this;
  }

}
