<?php

namespace Drupal\aegir_site_subscriptions\Wrappers;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Messenger\MessengerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\node\NodeInterface;
use Drupal\aegir_site_subscriptions\HostingServiceCalls\SiteQuotaHostingServiceCall;
use Drupal\aegir_site_subscriptions\HostingServiceCalls\SiteVerifyHostingServiceCall;

/**
 * Wrapper for site nodes providing additional functionality.
 */
class SiteWrapper extends Wrapper {

  /**
   * The site node.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $site;

  /**
   * Factory method.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container this instance should use.
   * @param \Drupal\node\NodeInterface $site
   *   The site node.
   *
   * @return \Drupal\aegir_site_subscriptions\Wrappers\SiteWrapper
   *
   * @throws \Exception
   *
   * @see ContainerInjectionInterface::create()
   */
  public static function create(ContainerInterface $container, NodeInterface $site) {
    return new static(
      $site,
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('entity_type.manager')->getStorage('node'),
      $container->get('config.factory')->get('aegir_site_subscriptions.settings'),
      $container->get('module_handler'),
      $container->get('logger.factory')->get('aegir_site_subscriptions'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\node\NodeInterface $site
   *   The site node.
   *
   * @throws \Exception
   */
  public function __construct(
    NodeInterface $site,
    Request $current_request = NULL,
    EntityStorageInterface $node_storage = NULL,
    ImmutableConfig $module_config = NULL,
    ModuleHandlerInterface $module_handler = NULL,
    LoggerInterface $logger = NULL,
    MessengerInterface $user_messenger = NULL
  ) {
    parent::__construct($current_request, $node_storage, $module_config, $module_handler, $logger, $user_messenger);

    if ($site->getType() != 'aegir_site') {
      throw new \Exception('A site node must be provided on construction.');
    }
    $this->site = $site;
  }

  /**
   * Fetches the site's subscription ID.
   *
   * @return string
   *   The subscription ID.
   */
  public function getSubscriptionId() {
    return $this->site->get('field_site_subscription_url')->getValue()[0]['title'];
  }

  /**
   * Fetches the subscription wrapper associated with the site.
   *
   * @return \Drupal\aegir_site_subscriptions\Wrappers\SubscriptionWrapper
   *   The subscription wrapper.
   */
  public function getSubscription() {
    return SubscriptionWrapper::getInstance($this->getSubscriptionId());
  }

  /**
   * Determines if a particular subscription is active for a site.
   *
   * @return bool
   *   TRUE if subscription is active; FALSE otherwise.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function subscriptionIsActive() {
    return SubscriptionWrapper::getInstance($this->getSubscriptionId())->isActive();
  }

  /**
   * Gets the configuration link for a specific site.
   *
   * @return \Drupal\Core\Link
   *   The site node's configuration link.
   */
  public function getConfigurationLink() {
    $url = Url::fromRoute('entity.node.edit_form', ['node' => $this->site->id()]);
    return Link::fromTextAndUrl(t('<em>Configuration required!</em>'), $url);
  }

  /**
   * Fetches the site's link.
   *
   * @return \Drupal\Core\Link
   *   A link to the site itself.
   */
  public function getLink() {
    $protocol = $this->currentRequest->getScheme();
    $subdomain = Html::escape($this->getTitle());
    $domain = $this->currentRequest->getHost();
    $hostname = "$subdomain.$domain";
    $url = Url::fromUri("$protocol://$hostname/");
    return Link::fromTextAndUrl($hostname, $url);
  }

  /**
   * Fetches the site's title.
   *
   * @return string
   *   The site's title.
   */
  public function getTitle() {
    return $this->site->getTitle();
  }

  /**
   * Saves the site node.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function save() {
    return $this->site->save();
  }

  /**
   * Set quotas based on the subscription plan & add-ons.
   *
   * @param string $plan_code
   *   The plan code for this site's subscription.
   * @param array $addons
   *   The list of add-ons included in the subscription.
   *
   * @see https://www.drupal.org/project/quenforcer
   *
   * @throws \Exception
   */
  public function setQuotas($plan_code, array $addons) {
    $http_client = \Drupal::service('http_client');

    $quotas = $this->moduleHandler->invokeAll('aegir_site_subscriptions_quota_info', [
      $this->site,
      $plan_code,
      $addons,
    ]);

    foreach ($quotas as $quota => $limit) {
      $quota_setter = new SiteQuotaHostingServiceCall(
        $this->logger,
        $http_client,
        $this->moduleConfig,
        $this->currentRequest,
        $this->moduleHandler,
        $this->userMessenger,
        $this->site,
        $quota,
        $limit
      );
      $quota_setter->performActionAndLogResults();
    }

    if (!empty($quotas)) {
      $site_verifier = new SiteVerifyHostingServiceCall(
        $this->logger,
        $http_client,
        $this->moduleConfig,
        $this->currentRequest,
        $this->moduleHandler,
        $this->userMessenger,
        $this->site
      );
      $site_verifier->performActionAndLogResults();
    }
  }

}
