<?php

namespace Drupal\aegir_site_subscriptions\Wrappers;

use Drupal\aegir_site_subscriptions\Plugin\SubscriptionProviderManager;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Messenger\MessengerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\node\Entity\Node;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Wrapper for subscriptions providing additional functionality.
 */
class SubscriptionWrapper extends Wrapper {

  /**
   * The subscription.
   *
   * @var object
   */
  protected $subscription;

  /**
   * The subscription provider manager.
   *
   * @var \Drupal\aegir_site_subscriptions\Plugin\SubscriptionProviderManager
   */
  protected $subscriptionProviderManager;

  /**
   * Factory method.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container this instance should use.
   * @param object $subscription
   *   The subscription.
   *
   * @return \Drupal\aegir_site_subscriptions\Wrappers\SubscriptionWrapper
   *   An instance of this class.
   *
   * @see ContainerInjectionInterface::create()
   */
  public static function create(ContainerInterface $container, $subscription) {
    return new static(
      $subscription,
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('entity_type.manager')->getStorage('node'),
      $container->get('config.factory')->get('aegir_site_subscriptions.settings'),
      $container->get('module_handler'),
      $container->get('logger.factory')->get('aegir_site_subscriptions'),
      $container->get('messenger'),
      $container->get('plugin.manager.aegir_site_subscription_provider')
    );
  }

  /**
   * Class Constructor.
   *
   * @param object $subscription
   *   The subscription.
   * @param \Symfony\Component\HttpFoundation\Request $current_request
   *   The current HTTP/S request.
   * @param \Drupal\Core\Entity\EntityStorageInterface $node_storage
   *   Node storage.
   * @param \Drupal\Core\Config\ImmutableConfig $module_config
   *   This module's configuration.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logging service.
   * @param \Drupal\Core\Messenger\MessengerInterface $user_messenger
   *   The module handler.
   * @param \Drupal\aegir_site_subscriptions\Plugin\SubscriptionProviderManager $subscription_provider_manager
   *   * The subscription provider manager.
   */
  public function __construct(
      $subscription,
      Request $current_request = NULL,
      EntityStorageInterface $node_storage = NULL,
      ImmutableConfig $module_config = NULL,
      ModuleHandlerInterface $module_handler = NULL,
      LoggerInterface $logger = NULL,
      MessengerInterface $user_messenger = NULL,
      SubscriptionProviderManager $subscription_provider_manager = NULL
  ) {
    parent::__construct($current_request, $node_storage, $module_config, $module_handler, $logger, $user_messenger);
    $this->subscription = $subscription;
    $this->subscriptionProviderManager = $subscription_provider_manager;
  }

  /**
   * Gets the subscription URL from an ID.
   *
   * @return string
   *   The subscription URL.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getUrl() {
    return $this->subscriptionProviderManager
      ->createInstance($this->getSubscriptionProviderPluginId())
      ->getSubscriptionUrl($this->getId());
  }

  /**
   * Creates a site record.
   *
   * @param int $user_id
   *   The ID of the user whose site is being created.
   *
   * @return \Drupal\node\Entity\Node
   *   The new site.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function createSite($user_id) {
    $subscription_id = $this->getId();

    $site = Node::create([
      'type' => 'aegir_site',
      // This is temporary until we get the user's preferred site name.
      'title' => $subscription_id,
      'uid' => $user_id,
      'field_site_subscription_url' => [
        'title' => $subscription_id,
        'uri' => $this->getUrl(),
      ],
    ]);

    $site->save();
    return $site;
  }

  /**
   * Fetches the list of add-ons.
   *
   * @return array
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   *
   * @see hook_aegir_site_subscriptions_quota_info()
   */
  public function getAddons() {
    return $this->subscriptionProviderManager
      ->createInstance($this->getSubscriptionProviderPluginId())
      ->getSubscriptionAddons($this->subscription);
  }

  /**
   * Fetches the list of site IDs associated with this subscription.
   *
   * @param int $user_id
   *   The local Drupal user ID.
   *
   * @return array
   *   The list of site IDs.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getSiteIds($user_id) {
    return $this->nodeStorage->getQuery()
      ->condition('type', 'aegir_site')
      ->condition('uid', $user_id)
      ->condition('field_site_subscription_url.title', $this->getId())
      ->execute();
  }

  /**
   * Factory method.
   *
   * @param string $subscription_id
   *   The subscription ID.
   *
   * @return \Drupal\aegir_site_subscriptions\Wrappers\SubscriptionWrapper|null
   *   An instance of this class if possible.
   */
  public static function getInstance($subscription_id) {
    if (empty($subscription_id)) {
      return NULL;
    }

    try {
      $subscription = \Drupal::service('plugin.manager.aegir_site_subscription_provider')
        ->createInstance(static::getSubscriptionProviderPluginId())
        ->getSubscriptionInstance($subscription_id);
    }
    catch (\Exception $e) {
      return NULL;
    }

    return is_object($subscription) ? static::create(\Drupal::getContainer(), $subscription) : NULL;
  }

  /**
   * Fetches the ID.
   *
   * @return string|null
   *   The ID or NULL if there isn't one.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getId() {

    return $this->subscriptionProviderManager
      ->createInstance($this->getSubscriptionProviderPluginId())
      ->getSubscriptionId($this->subscription);
  }

  /**
   * Returns a Drupal user ID corresponding with one in the subscription system.
   *
   * @param string $subscription_provider_user_id
   *   The user ID in the subscriptions system.
   *
   * @return int
   *   The Drupal user ID.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getDrupalUserId($subscription_provider_user_id) {
    return $this->subscriptionProviderManager
      ->createInstance($this->getSubscriptionProviderPluginId())
      ->getDrupalUserId($subscription_provider_user_id);
  }

  /**
   * Fetches a site corresponding to the subscription if it's active.
   *
   * @param string $account_code
   *   The remote account code.
   *
   * @return \Drupal\node\Entity\Node|null
   *   The site node or NULL if there isn't a site with an active subscription.
   *
   * @throws \Exception
   */
  public function getSiteIfSubscriptionIsActive($account_code) {
    if (($site = $this->getSite($account_code)) &&
        (new SiteWrapper($site))->subscriptionIsActive()) {
      return $site;
    }
    return NULL;
  }

  /**
   * Fetches a site corresponding to the subscription.
   *
   * @param string $account_code
   *   The remote account code.
   *
   * @return \Drupal\node\Entity\Node|null
   *   The site node or NULL if there isn't an associated site.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getSite($account_code) {
    $site_ids = $this->getSiteIds($this->getDrupalUserId($account_code));
    if (empty($site_ids) || (!$site = Node::load(array_pop($site_ids)))) {
      return NULL;
    }
    return $site;
  }

  /**
   * Fetch the plan code.
   *
   * @return string
   *   The plan code.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getPlanCode() {
    return $this->subscriptionProviderManager
      ->createInstance($this->getSubscriptionProviderPluginId())
      ->getSubscriptionPlanCode($this->subscription);
  }

  /**
   * Determines if the subscription is active or not.
   *
   * @return bool
   *   TRUE if the subscription is active; FALSE otherwise.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function isActive() {
    return $this->subscriptionProviderManager
      ->createInstance($this->getSubscriptionProviderPluginId())
      ->subscriptionIsActive($this->subscription);
  }

  /**
   * Fetches the plugin ID for the active subscription provider.
   *
   * @return string
   */
  protected static function getSubscriptionProviderPluginId() {
    if (!$provider_plugin_id = \Drupal::config('aegir_site_subscriptions.settings')->get('subscription_provider')) {
      // @todo Report emergency: no subscription provider selected.
    }
    return $provider_plugin_id;
  }

}
